from mock import MagicMock
from pymacaron_core.swagger.apipool import ApiPool
from pymacaron_core.swagger.api import API


yaml_foo = """
swagger: '2.0'
info:
  title: test
  version: '0.0.1'
  description: Just a test
host: some.server.com
schemes:
  - http
basePath: /v1
produces:
  - application/json
paths:
  /v1/foo:
    post:
      parameters:
        - in: query
          type: string
          name: arg1
          description: foooo
          required: true
      x-bind-server: pymacaron_core.test.return_token
      produces:
        - application/json
      responses:
        '200':
          description: result
          type: string
"""

yaml_bar = """
swagger: '2.0'
info:
  title: test
  version: '0.0.1'
  description: Just a test
host: another.server.com
schemes:
  - http
basePath: /v1
produces:
  - application/json
paths:
  /v1/bar:
    post:
      parameters:
        - in: query
          type: string
          name: arg1
          description: foooo
          required: true
      x-bind-server: pymacaron_core.test.return_token
      produces:
        - application/json
      responses:
        '200':
          description: result
          type: string
"""

def test_apipool_add():
    assert not hasattr(ApiPool, 'foo')
    assert not hasattr(ApiPool, 'bar')

    ApiPool().add('foo', yaml_str=yaml_foo)
    assert hasattr(ApiPool, 'foo')
    assert not hasattr(ApiPool, 'bar')
    assert isinstance(ApiPool().foo, API)

    ApiPool().add('bar', yaml_str=yaml_foo)
    assert hasattr(ApiPool, 'foo')
    assert hasattr(ApiPool, 'bar')
    assert isinstance(ApiPool().bar, API)

def test_apipool_current_server_name_api():
    assert ApiPool().current_server_name == ''
    assert ApiPool().current_server_api is None

    app = MagicMock()
    api = ApiPool().add('foo', yaml_str=yaml_foo)
    api.spawn_api(app)

    assert ApiPool().current_server_name == 'foo'
    assert ApiPool().current_server_api == api

def test__cmp_models():
    assert ApiPool._cmp_models(
        {'a': 1, 'b': 2},
        {'a': 1, 'b': 2}
    ) == 0

    assert ApiPool._cmp_models(
        {'a': 1},
        {'a': 1, 'b': 2}
    ) != 0

    assert ApiPool._cmp_models(
        {'a': 1, 'b': 2, 'x-model': 12},
        {'a': 1, 'b': 2}
    ) == 0

    assert ApiPool._cmp_models(
        {'a': 1, 'b': 2, 'x-model': 12, 'properties': {'foo': {'$ref': 'a'}}},
        {'a': 1, 'b': 2, 'properties': {'foo': {'$ref': 'a', 'x-scope': [1, 2]}}},
    ) == 0



import unittest
from mock import patch
from pymacaron_core.swagger.api import API


#
# Swagger spec
#

yaml = """
swagger: '2.0'
info:
  version: '0.0.1'
host: some.server.com
schemes:
  - http
produces:
  - application/json
definitions:
  Foo:
    type: object
    description: a foo
    x-persist: pymacaron_core.test.PersistentFoo
    properties:
      foo:
        type: string
        format: foo
        description: bar
"""

#
# Tests
#

class Tests(unittest.TestCase):

    def test_x_persist(self):
        api = API('somename', yaml_str=yaml)

        Foo = api.model.Foo
        self.assertTrue(hasattr(Foo, 'load_from_db'))
        self.assertEqual(Foo.load_from_db(), 'bob')

        f = Foo()
        self.assertTrue(hasattr(f, 'save_to_db'))
        self.assertEqual(f.save_to_db(), 'foo')



import unittest
from pymacaron_core.swagger.api import API


#
# Swagger spec
#

yaml_str = """
swagger: '2.0'
info:
  version: '0.0.1'
host: some.server.com
schemes:
  - http
produces:
  - application/json
definitions:
  Foo:
    type: object
    description: a foo
    x-parent: pymacaron_core.test.FunnyDad
    properties:
      foo:
        type: string
        format: foo
        description: bar
"""

#
# Tests
#

class Tests(unittest.TestCase):

    def test_x_parent(self):
        api = API('somename', yaml_str=yaml_str)

        Foo = api.model.Foo
        f = Foo()
        self.assertTrue(hasattr(f, 'lol'))
        self.assertTrue(hasattr(f, 'roflol'))
        self.assertEqual(f.lol(), 'lol')
        self.assertEqual(f.roflol(), 'roflol')



import unittest
from jsonschema.exceptions import ValidationError
from bravado_core.formatter import SwaggerFormat
from bravado_core.exception import SwaggerValidationError
from bravado_core.unmarshal import unmarshal_model
from bravado_core.validate import validate_object
from pymacaron_core.swagger.api import API

formats = []

#
# Stripe Card Token
#

def validate_foo(foo):
    if foo != 'foo':
        raise SwaggerValidationError("Foo is not foo")


foo_format = SwaggerFormat(
    format='foo',
    to_wire=lambda s: s,
    to_python=lambda s: s,
    validate=validate_foo,
    description='a foo'
)

#
# Swagger spec
#

yaml_str = """
swagger: '2.0'
info:
  version: '0.0.1'
host: some.server.com
schemes:
  - http
produces:
  - application/json
definitions:
  Foo:
    type: object
    description: a foo
    properties:
      foo:
        type: string
        format: foo
        description: bar
"""

#
# Tests
#

class Tests(unittest.TestCase):

    def test_custom_format(self):
        api = API('somename', yaml_str=yaml_str, formats=[foo_format])

        self.assertTrue(hasattr(api.model, 'Foo'))

        # marshal and unmarshal
        f = api.model.Foo(foo='123')
        j = api.api_spec.model_to_json(f)
        self.assertDictEqual(j, {'foo': '123'})

        model_def = api.api_spec.swagger_dict['definitions']['Foo']
        f = unmarshal_model(api.api_spec.spec, model_def, j)
        self.assertEqual(f.foo, '123')

        # validate
        validate_object(api.api_spec.spec, model_def, {'foo': 'foo'})

        try:
            validate_object(api.api_spec.spec, model_def, {'foo': '123'})
        except ValidationError as e:
            self.assertTrue("'123' is not a 'foo'" in str(e))
        else:
            assert 0
